#!/bin/bash
#
# prepare-raspbian-sd:bin/api/raspberrypi-org-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function echofatal not found.' >&2
  exit 100
fi
if [ ! -d "${RASPBIAN_API_HOME:-}" ] ; then
  echofatal "RASPBIAN_API_HOME must be define and valid: '${RASPBIAN_API_HOME:-}'"
  exit 100
fi

# return: 'armhf' or 'arm64'
function raspberrypi_org_get_valid_arch_for_url {
  case "${arch_type}" in
    'armhf'|'arm32'|'armv7l')
       echo 'armhf'
       ;;

    'arm64'|'aarch64')
       echo 'arm64'
       ;;

    '')
       # Support orginal raspbian
       ;;

    *)
      echofatal "${FUNCNAME[0]}() - Unsupported arch type: '${arch_type}'"
      return 100
      ;;
  esac
}

function raspberrypi_org_get_url_raspbian_last_version {
  local -r os_type="$1"

  echodebug "${FUNCNAME[0]}()"

  local __custom__=
  case "${os_type}" in
    '--lite'            ) __custom__='lite_' ;;
    '--full-desktop'    ) __custom__='full_' ;;
    '--minimal-desktop' ) __custom__='' ;;
    *)
      echofatal "${FUNCNAME[0]}( '${os_type}' ) - Wrong os_type"
      return 22
      ;;
  esac
  local -r __custom__

  local -r url="https://downloads.raspberrypi.org/raspbian_${__custom__}latest"

  echoinfo "Compute last version using based on os_type='${os_type}' : '${url}'"

# curl --head "${url}" |
#   grep -i '^Location: ' |
#   cut -d ' ' -f2- |
#   tr '\r' '\n' |
#   head -n 1

  curl --head "${url}" |
    sed -E -e '/Location:/I!d' -e 's,Location:[[:blank:]](.*),\1,gI' |
    tr '\r' '\n' |
    head -n 1
}

function raspberrypi_org_get_url_raspios_last_version {
  local -r arch_type="$1"
  local -r os_type="$2"

  echodebug "${FUNCNAME[0]}( arch_type='${arch_type}' os_type='${os_type}' )"

  local identifier=
  case "${os_type}" in
    '--lite')
      identifier="lite_${arch_type}_latest"
      ;;

    '--full-desktop')
      identifier="full_${arch_type}_latest"
      ;;

    '--minimal-desktop')
      identifier="${arch_type}_latest"
      ;;

    *)
      echofatal "${FUNCNAME[0]}( '${os_type}' ) - Wrong os_type"
      return 23
      ;;
  esac
  local -r url="https://downloads.raspberrypi.org/raspios_${identifier}"

  echoinfo "Compute last version using based on os_type='${os_type}'/arch_type='${arch_type}' : '${url}'"

# curl --head "${url}" |
#   grep -i '^Location: ' |
#   cut -d ' ' -f2- |
#   tr '\r' '\n' |
#   head -n 1

  curl --head "${url}" |
    sed -E -e '/Location:/I!d' -e 's,Location:[[:blank:]](.*),\1,gI' |
    tr '\r' '\n' |
    head -n 1
}

#
# Sample:
#   raspberrypi_org_get_json 'buster' 'arm64' '--full-desktop'
#
function raspberrypi_org_get_json {
  local -r os_name="$1"     # 'buster' or 'bullseye'
  local -r arch_type="$2"   # 'armhf', 'arm32', '', 'armv7l' or 'arm64', 'aarch64'
  local -r os_type="$3"

  echodebug "${FUNCNAME[0]}()"

  local arch_type_for_url= # 'armhf' or 'arm64'
  arch_type_for_url="$( raspberrypi_org_get_valid_arch_for_url "${arch_type}" )" || return $?
  local -r arch_type_for_url # 'armhf' or 'arm64'

  local identifier=
  case "${os_type}" in
    '--lite')
      identifier="-${arch_type_for_url}-lite.img"
      ;;

    '--full-desktop')
      identifier="full_${arch_type_for_url}.img"
      ;;

    '--minimal-desktop')
      identifier="${arch_type_for_url}.img"
      ;;

    *)
      echofatal "${FUNCNAME[0]}( '${os_type}' ) - Wrong os_type"
      return 23
      ;;
  esac

  local remote_json=
  remote_json="$(
    [ "${DEBUG:-}" = 'true' ] && set -x
    curl --silent --location 'https://downloads.raspberrypi.org/os_list_imagingutility_v4.json'
    )" || return $?
  local -r remote_json

  local flattern_json=
  flattern_json="$(
    [ "${DEBUG:-}" = 'true' ] && set -x
    jq -c -f "${RASPBIAN_API_HOME}/os_list_imagingutility_v4.jq" <<<"${remote_json}"
  )"
  local -r flattern_json

  echodebug "flattern_json='${flattern_json}'"

  local filtered_json=
  filtered_json="$(
    [ "${DEBUG:-}" = 'true' ] && set -x
    jq -c <<<"${flattern_json}" \
        --arg osname "-${os_name}-" \
        --arg identifier "${identifier}" \
        'map( select( .url | contains( $osname ) ) | select( .url | contains( $identifier ) ) )'
  )" || return $?

  echodebug "filtered_json='${filtered_json}'"

  if [ "${filtered_json}" = '[]' ] ; then
    echofatal "Can't find entry for with osname='${os_name}' and  identifier='${identifier}'"

    echoerror "Results with os_name: '${os_name}'"
#   grep --color -i "${os_name}" <<<"${remote_json}" >&2
    jq -r <<<"${flattern_json}" --arg osname "${os_name}" 'map( select( .url | contains( $osname ) ) | {name,description,url} )' >&2

    echoerror "Results with arch: '${arch_type_for_url}'"
#   grep --color -i "${arch_type_for_url}" <<<"${remote_json}" >&2
    jq -r <<<"${flattern_json}" --arg arch "${arch_type_for_url}" 'map( select( .url | contains( $arch ) ) | {name,description,url} )' >&2

    return 100
  fi

  (
    [ "${DEBUG:-}" = 'true' ] && set -x
    jq -c '.[0]' <<<"${filtered_json}" # return first entry
  ) || return $?
}

