#!/bin/bash
#
# prepare-raspbian-sd:bin/prepare-raspbian-sd-config.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -gr PROJECT_HOME

declare -gr APIS_HOME="${PROJECT_HOME}/apis"
declare -gr RASPBIAN_API_HOME="${PROJECT_HOME}/bin/api"

# shellcheck disable=SC1091
source "${APIS_HOME}/bash-common-api/bash-common-api.sh" || exit $?
# shellcheck disable=SC1091
source "${RASPBIAN_API_HOME}/prepare-raspbian-api.sh" || exit $?
# shellcheck disable=SC1091
source "${RASPBIAN_API_HOME}/prepare-raspbian-config-json-api.sh" || exit $?

function usage {
  cat <<EOF
Configure prepare-raspbian-sd, allow to modify '$( prepare_raspbian_get_config_filename )'

Usage:

   $( basename "$0" ) --help
   $( basename "$0" ) --get VARNAME
   $( basename "$0" ) --list
   $( basename "$0" ) --list-all
   $( basename "$0" ) --set-boolean VARNAME VALUE
   $( basename "$0" ) --set-integer VARNAME VALUE
   $( basename "$0" ) --set-link VARNAME LINK_TARGET
   $( basename "$0" ) [--set] VARNAME VALUE
   $( basename "$0" ) --show-filename

Actions:
  -h, --help                Display help message and exit.
      --get VARNAME         Show given variable content.
  -l, --list                Display valid variables name's in JSON format.
  -L, --list-all            Show all define variables in JSON format.
      --set-boolean VARNAME VALUE
                            Set given VALUE to variable VARNAME. VALUE must be a boolean (true|false).
      --set-integer VARNAME VALUE
                            Set given VALUE to variable VARNAME. VALUE must be an integer.
      --set-link VARNAME LINK_TARGET
                            The value of the VARNAME field corresponds to the content of the file
                            defined by LINK_TARGET. The associated value will be the raw content
                            of this file.
      --set VARNAME VALUE   Set given VALUE to variable VARNAME.
      --show-filename       Show configuration filename and exit.

EOF
}

function test_action {
  if [ -n "${action}" ] ; then
    echofatal "Action already set to '${action}'"
    usage >&2
    exit 1
  fi
}

function main {
  local action=
  local param=
  local param_name='_WRONG_NAME_'
  local param_value='_WRONG_VALUE_'

  if [ "$#" -eq 0 ] ; then
    echofatal 'Parameters missing.'
    usage >&2
    return 100
  fi

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        return 0
        ;;

      '-l'|'--list')
        test_action && action='--list'
        ;;

      '-L'|'--list-all')
        test_action && action='--list-all'
        ;;

      '--get')
        test_action && action='--get'
        param_name="$( P_param "$@" )" && shift || return $?
        ;;

      '--set')
        test_action && action='--set'
        param_name="$( P_param "$@" )" && shift || return $?
        param_value="$( P_param "$@" )" && shift || return $?
        ;;

      '--set-boolean'|'--set-integer')
        test_action && action="${param}"
        param_name="$( P_param "$@" )" && shift || return $?
        param_value="$( P_param "$@" )" && shift || return $?
        ;;

      '--set-link')
        test_action && action='--set-link'
        param_name="$( P_param "$@" )" && shift || return $?
        param_value="$( P_param "$@" )" && shift || return $?
        ;;

      '--show-filename')
        test_action && action='--show-filename'
        ;;

      *)
        test_action && action='--set'
        param_name="${param}"
        param_value="$( P_param "$@" )" && shift || return $?
        ;;
    esac
  done

  case "${action}" in
    '--list')
      prepare_raspbian_show_list_json || return $?
      ;;

    '--list-all')
      prepare_raspbian_show_list_all || return $?
      ;;

    '--set')
      prepare_raspbian_check_and_set_value 'string' "${param_name}" "${param_value}" || return $?
      ;;

    '--set-boolean')
      prepare_raspbian_check_and_set_value 'boolean' "${param_name}" "${param_value}" || return $?
      ;;

    '--set-integer')
      prepare_raspbian_check_and_set_value 'number' "${param_name}" "${param_value}" || return $?
      ;;

    '--set-link')
      prepare_raspbian_check_and_set_value 'LINK' "${param_name}" "${param_value}" || return $?
      ;;

    '--get')
      prepare_raspbian_check_and_get_value "${param_name}" || return $?
      ;;

    '--show-filename')
      prepare_raspbian_show_filename || return $?
      ;;

    *)
      echofatal "Internal error: action='${action}'"
      return 100
      ;;
  esac
}

main "$@" || exit $?
